import requests
from python_socialite.drivers.abstract_driver import AbstractDriver
from python_socialite.drivers.abstract_user import abstract_user


class GoogleProvider(AbstractDriver):
    def __init__(self, config):
        """Initialize Google provider."""
        super().__init__(config)
        self.scopes = config.get("scopes", ["openid", "email", "profile"])

    def get_auth_url(self):
        url = "https://accounts.google.com/o/oauth2/v2/auth"
        return self.build_url(url)

    def get_token_url(self):
        return "https://www.googleapis.com/oauth2/v4/token"

    def get_token_fields(self, code):
        fields = super().get_token_fields(code)
        fields["grant_type"] = "authorization_code"
        return fields

    def get_user_by_token(self, access_token):
        url = "https://www.googleapis.com/oauth2/v3/userinfo"
        headers = {"Authorization": f"Bearer {access_token}"}
        response = requests.get(url, headers=headers)
        response.raise_for_status()
        return response.json()

    def map_user_to_dict(self, raw_user):
        user = dict(abstract_user)
        user["id"] = raw_user.get("sub")
        user["name"] = raw_user.get("name")
        user["email"] = raw_user.get("email")
        user["avatar"] = raw_user.get("picture")
        user["provider"] = "google"
        user["raw"] = raw_user
        return user



"""Main module."""
from python_socialite.drivers.google import GoogleProvider


class OAuthProvider:
    def __init__(self, driver, config):
        """Initialize default provider."""
        credentials = config.get(driver)
        if driver == "google":
            self.provider = GoogleProvider(credentials)
        else:
            raise ValueError("Invalid socialite driver")

    def set_scopes(self, scopes):
        self.provider.set_scopes(scopes)
        return self

    def get_auth_url(self):
        return self.provider.get_auth_url()

    def get_token(self, code):
        return self.provider.get_token(code)

    def get_user(self, access_token):
        return self.provider.get_user(access_token)



#!/usr/bin/env python
# fmt: off
import codecs
import os.path
from setuptools import setup, find_packages  # type: ignore


def read(rel_path):
    here = os.path.abspath(os.path.dirname(__file__))
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


with open("README.md") as readme_file:
    readme = readme_file.read()

setup(
    author="Evans Mwendwa",
    author_email="evans@authenticvisualsmedia.com",
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 2 - Pre-Alpha",
        "License :: OSI Approved :: MIT License",
        "Natural Language :: English",
        "Programming Language :: Python :: 3",
    ],
    description="Simple Python oAuth Login",
    install_requires=["requests"],
    license="MIT license",
    long_description=readme,
    long_description_content_type="text/markdown",
    package_data={"python_socialite": ["py.typed"]},
    include_package_data=True,
    keywords="python_socialite",
    name="python_socialite",
    package_dir={"": "src"},
    packages=find_packages(
        include=["src/python_socialite", "src/python_socialite.*"]
    ),
    setup_requires=[],
    url="https://github.com/evansmwendwa/python-socialite",
    version=get_version("src/python_socialite/__init__.py"),
    zip_safe=False,
)
# fmt: on



import requests
from abc import ABCMeta, abstractmethod
from urllib import parse


class AbstractDriver(metaclass=ABCMeta):
    def __init__(self, config):
        """Initialize default attributes."""
        self.client_id = config.get("client_id")
        self.client_secret = config.get("client_secret")
        self.redirect_url = config.get("redirect_url")
        self.config = config
        self.scope_separator = " "

    @abstractmethod
    def get_auth_url(self):
        raise NotImplementedError

    @abstractmethod
    def get_token_url(self):
        raise NotImplementedError

    def set_scopes(self, scopes):
        self.scopes = scopes

    def build_scope(self):
        delimeter = self.scope_separator
        return delimeter.join(self.scopes)

    def get_code_fields(self, state=None):
        fields = {
            "client_id": self.client_id,
            "redirect_uri": self.redirect_url,
            "scope": self.build_scope(),
            "response_type": "code",
            "response_mode": "query",
        }

        if state is not None:
            fields["state"] = state

        return fields

    def get_token_fields(self, code):
        fields = {
            "client_id": self.client_id,
            "client_secret": self.client_secret,
            "redirect_uri": self.redirect_url,
            "code": code,
        }

        return fields

    def build_url(self, url, state=None):
        fields = self.get_code_fields(state)
        parts = list(parse.urlparse(url))
        query = dict(parse.parse_qsl(parts[4]))
        query.update(fields)
        parts[4] = parse.urlencode(query, quote_via=parse.quote)

        return parse.urlunparse(parts)

    def get_token(self, code):
        url = self.get_token_url()
        data = self.get_token_fields(code)
        response = requests.post(url, json=data)
        response.raise_for_status()
        return response.json()

    @abstractmethod
    def get_user_by_token(self, access_token):
        raise NotImplementedError

    @abstractmethod
    def map_user_to_dict(self, raw_user):
        raise NotImplementedError

    def get_user(self, access_token):
        raw_user = self.get_user_by_token(access_token)
        user = self.map_user_to_dict(raw_user)
        return user
